@@ -47,6 +47,7 @@ namespace kt
 		vbox->setMargin(0);
 		vbox->setSpacing(0);
 		view = new QTreeView(this);
+		view->setUniformRowHeights(true);
 		toolbar = new QToolBar(this);
 		toolbar->setOrientation(Qt::Vertical);
 		toolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
@@ -60,7 +61,7 @@ namespace kt
 		search->hide();
 		vbox->addWidget(search);
 		vbox->addWidget(view);
-		layout->addItem(vbox);
+		layout->addLayout(vbox);
 		
 		show_search = toolbar->addAction(KIcon("edit-find"),i18n("Show Search"));
 		show_search->setToolTip(i18n("Show or hide the search bar"));
