@@ -23,7 +23,9 @@
 #include <kiconloader.h>
 #include <QVBoxLayout>
 #include <KIcon>
+#include <KToolBar>
 #include <QAction>
+#include <QTimer>
 
 namespace kt
 {
@@ -71,8 +73,9 @@ namespace kt
 		QVBoxLayout* layout = new QVBoxLayout(this);
 		layout->setSpacing(0);
 		layout->setMargin(0);
-		tab_bar = new QToolBar(this);
+		tab_bar = new KToolBar(this);
 		tab_bar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+		connect(tab_bar,SIGNAL(toolButtonStyleChanged(Qt::ToolButtonStyle)),this,SLOT(toolButtonStyleChanged(Qt::ToolButtonStyle)));
 		action_group = new ActionGroup(this);
 		connect(action_group,SIGNAL(actionTriggered(QAction*)),this,SLOT(onActionTriggered(QAction*)));
 		widget_stack = new QStackedWidget(splitter);
@@ -100,7 +103,7 @@ namespace kt
 		QAction* act = tab_bar->addAction(KIcon(icon),text);
 		act->setCheckable(true);
 		act->setToolTip(tooltip);
-		act->setChecked(false);
+		act->setChecked(widget_stack->count() == 0);
 		widget_stack->addWidget(ti);
 		action_group->addAction(act);
 		widget_to_action.insert(ti,act);
@@ -134,6 +137,15 @@ namespace kt
 			widget_stack->hide();
 			hide();
 		}
+		else
+		{
+			QMap<QWidget*,QAction*>::iterator itr = widget_to_action.find(widget_stack->currentWidget());
+			if (itr != widget_to_action.end())
+			{
+				QAction* act = itr.value();
+				act->setChecked(true);
+			}
+		}
 	}
 	
 	void TabBarWidget::changeTabIcon(QWidget* ti,const QString & icon)
@@ -218,6 +230,7 @@ namespace kt
 			if (i.value()->text() == ctab)
 			{
 				widget_stack->setCurrentWidget(i.key());
+				i.value()->setChecked(true);
 				break;
 			}
 		}
@@ -231,4 +244,17 @@ namespace kt
 				unshrink();
 		}
 	}
+	
+	void TabBarWidget::toolButtonStyleChanged(Qt::ToolButtonStyle style)
+	{
+		if (style != Qt::ToolButtonTextBesideIcon)
+			QTimer::singleShot(0,this,SLOT(setToolButtonStyle()));
+	}
+	
+	void TabBarWidget::setToolButtonStyle()
+	{
+		tab_bar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+	}
+
+
 }
\ No newline at end of file
