@@ -41,6 +41,8 @@ namespace kt
 		model = new WeekDayModel(this);
 		m_day_list->setModel(model);
 		
+		connect(m_day_list,SIGNAL(clicked(QModelIndex)),this,SLOT(dayClicked(QModelIndex)));
+		
 		connect(m_from,SIGNAL(timeChanged(const QTime & )),this,SLOT(fromChanged(const QTime&)));
 		connect(m_to,SIGNAL(timeChanged(const QTime & )),this,SLOT(toChanged(const QTime&)));
 		connect(m_entire_week,SIGNAL(clicked()),this,SLOT(selectEntireWeek()));
@@ -50,7 +52,9 @@ namespace kt
 		setWindowTitle(i18n("Add an item"));
 		
 		m_from->setTime(QTime(10,0,0));
+		m_from->setMaximumTime(QTime(23,58,0));
 		m_to->setTime(QTime(11,59,59));
+		m_to->setMinimumTime(QTime(0,1,0));
 	
 		m_suspended->setChecked(false);
 		m_upload_limit->setValue(0);
@@ -120,13 +124,15 @@ namespace kt
 	void AddItemDlg::fromChanged(const QTime & time)
 	{
 		// ensure that from is always smaller then to
-		m_to->setMinimumTime(time.addSecs(60));
+		if (time >= m_to->time())
+			m_to->setTime(time.addSecs(60));
 	}
 	
 	void AddItemDlg::toChanged(const QTime & time)
 	{
 		// ensure that from is always smaller then to
-		m_from->setMaximumTime(time.addSecs(-60));
+		if (time <= m_from->time())
+			m_from->setTime(time.addSecs(-60));
 	}
 	
 	void AddItemDlg::selectEntireWeek()
@@ -176,6 +182,15 @@ namespace kt
 		m_ss_download_limit->setEnabled(!m_suspended->isChecked() && on);
 		m_ss_upload_limit->setEnabled(!m_suspended->isChecked() && on);
 	}
+	
+	void AddItemDlg::dayClicked(const QModelIndex& idx)
+	{
+		if (model->data(idx,Qt::CheckStateRole) != Qt::Checked)
+			model->setData(idx,Qt::Checked,Qt::CheckStateRole);
+		else
+			model->setData(idx,Qt::Unchecked,Qt::CheckStateRole);
+	}
+
 }
 
 #include "additemdlg.moc"
