@@ -50,6 +50,7 @@ namespace kt
 	{
 		bt::Out(SYS_SNF|LOG_NOTICE) << "ScanFolder : scanning " << dir << endl;
 		m_dir = new KDirLister();
+		m_dir->setAutoErrorHandlingEnabled(false,0);
 		
 		if (!m_root_dir.endsWith(bt::DirSeparator()))
 			m_root_dir += bt::DirSeparator();
@@ -139,24 +140,45 @@ namespace kt
 					m_incomplePollingTimer.start(10000);
 				}
 			}
-			else
+			else if (!m_to_load.contains(source) && !m_pendingURLs.contains(source))
 			{
 				bt::Out(SYS_SNF|LOG_NOTICE) << "ScanFolder : found " << source << endl;
-				//Add pending entry...
-				m_pendingURLs.push_back(source);
-				
-				QString group;
-				if (ScanFolderPluginSettings::addToGroup())
-					group = ScanFolderPluginSettings::group();
-				//Load torrent
-				if (ScanFolderPluginSettings::openSilently())
-					m_core->loadSilently(source,group);
-				else
-					m_core->load(source,group);
+				m_to_load.append(source);
 			}
 		}
+		
+		loadDelayed();
 	}
 	
+	void ScanFolder::loadDelayed()
+	{
+		// Don't load to many in a row
+		int loaded = 0;
+		KUrl::List::iterator i = m_to_load.begin();
+		while (i != m_to_load.end() && loaded < 10)
+		{
+			KUrl source = *i;
+			//Add pending entry...
+			m_pendingURLs.push_back(source);
+			QString group;
+			if (ScanFolderPluginSettings::addToGroup())
+				group = ScanFolderPluginSettings::group();
+		
+			//Load torrent
+			if (ScanFolderPluginSettings::openSilently())
+				m_core->loadSilently(source,group);
+			else
+				m_core->load(source,group);
+				
+			i = m_to_load.erase(i);
+			loaded++;
+		}
+		
+		if (!m_to_load.isEmpty())
+			QTimer::singleShot(2000, this, SLOT(loadDelayed()));
+	}
+
+	
 	void ScanFolder::onLoadingFinished(const KUrl & url, bool success, bool canceled)
 	{
 		Q_UNUSED(canceled);
@@ -195,7 +217,7 @@ namespace kt
 					QFile::remove(dirname + "." + name);
 
 				// NetAccess considered harmfull !!!
-				KIO::file_move(url, destination);
+				KIO::file_move(url, destination, -1, KIO::HideProgressInfo);
 				break;
 			case defaultAction:
 				QFile f(dirname + "." + name);
@@ -259,6 +281,7 @@ namespace kt
 	
 	void ScanFolder::onIncompletePollingTimeout()
 	{
+		QMap<KUrl,QString> todo;
 		bt::Out(SYS_SNF|LOG_NOTICE) << "ScanFolder : checking incomplete files" << endl; 
 		for (QList<KUrl>::iterator i = m_incompleteURLs.begin(); i != m_incompleteURLs.end();)
 		{
@@ -278,12 +301,8 @@ namespace kt
 				if (ScanFolderPluginSettings::addToGroup())
 					group = ScanFolderPluginSettings::group();
 				
-				//Load torrent
-				if (ScanFolderPluginSettings::openSilently())
-					m_core->loadSilently(source,group);
-				else
-					m_core->load(source,group);
-				
+				// don't load directly to avoid nested eventloops
+				todo.insert(source,group);
 				// remove from incomplete list
 				i = m_incompleteURLs.erase(i);
 			}
@@ -297,6 +316,16 @@ namespace kt
 		// stop timer when no incomple URL's are left
 		if (m_incompleteURLs.count() == 0)
 			m_incomplePollingTimer.stop();
+		
+		QMap<KUrl,QString>::iterator i = todo.begin();
+		while (i != todo.end())
+		{
+			if (ScanFolderPluginSettings::openSilently())
+				m_core->loadSilently(i.key(),i.value());
+			else
+				m_core->load(i.key(),i.value());
+			i++;
+		}
 	}
 }
 
