@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Joris Guisson                                   *
+ *   Copyright (C) 2005-2007 by Joris Guisson                              *
  *   joris.guisson@gmail.com                                               *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -15,18 +15,21 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  ***************************************************************************/
 #ifndef BTSEARCHWIDGET_H
 #define BTSEARCHWIDGET_H
 
 #include <qwidget.h>
-#include <qvaluevector.h>
 #include <kurl.h>
+#include <ktoolbar.h>
+#include <klineedit.h>
+#include "webview.h"
 
-class SearchBar;
-
-class KPopupMenu;
+class QProgressBar;
+class QNetworkReply;
+class KMenu;
+class KComboBox;
 
 namespace KParts
 {
@@ -35,50 +38,63 @@ namespace KParts
 
 namespace kt
 {
-	class HTMLPart;
+	class SearchWidget;
 	class SearchPlugin;
-	class SearchEngineList;
 	
 	
 	/**
-	@author Joris Guisson
+		@author Joris Guisson
+		
+		Widget which shows a KHTML window with the users search in it
 	*/
-	class SearchWidget : public QWidget
+	class SearchWidget : public QWidget,public WebViewClient
 	{
 		Q_OBJECT
 	public:
 		SearchWidget(SearchPlugin* sp);
 		virtual ~SearchWidget();
-	
-		KPopupMenu* rightClickMenu();
 		
-		void updateSearchEngines(const SearchEngineList & sl);
+		QString getSearchText() const {return search_text->text();}
+		KUrl getCurrentUrl() const;
+		QString getSearchBarText() const;
+		int getSearchBarEngine() const;
+		void setSearchBarEngine(int engine);
+		bool backAvailable() const;
+		void restore(const KUrl & url,const QString & text,const QString & sb_text,int engine);
+		
+	signals:
+		void enableBack(bool on);
+		void openNewTab(const KUrl & url);
+		void changeTitle(SearchWidget* w,const QString & title);
+		void changeIcon(SearchWidget* w,const QIcon & icon);
 	
 	public slots:
 		void search(const QString & text,int engine = 0);
-		void copy();
-		void onShutDown();
+		void home();
+		void search();
 	
 	private slots:
-		void searchPressed();
-		void clearPressed();
-		void onURLHover(const QString & url);
-		void onFinished();
-		void onOpenTorrent(const KURL & url);
-		void onSaveTorrent(const KURL & url);
-		void showPopupMenu(const QString & s,const QPoint & p);
-		void onBackAvailable(bool available);
-		void onFrameAdded(KParts::Part* p);
-		void statusBarMsg(const QString & url);
-		void openTorrent(const KURL & url);
-	
+		void loadStarted();
+		void loadFinished(bool ok);
+		void loadProgress(int p);
+		void unsupportedContent(QNetworkReply* reply);
+		void torrentDownloadFinished();
+		void iconChanged();
+		void titleChanged(const QString & text);
 		
 	private:
-		HTMLPart* html_part;
-		SearchBar* sbar;
-		KPopupMenu* right_click_menu;
-		int back_id;
+		virtual KUrl searchUrl(const QString& search_text);
+		virtual QWebView* newTab();
+		
+	private:
+		WebView* webview;
+		KToolBar* sbar;
 		SearchPlugin* sp;
+		QProgressBar* prog;
+		QNetworkReply* torrent_download;
+		
+		KComboBox* search_engine;
+		KLineEdit* search_text;
 	};
 
 }
