@@ -15,12 +15,17 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  ***************************************************************************/
+#include <time.h>
 #include <QDir>
+#include <QRegExp>
 #include <QSocketNotifier>
-#include <qtimer.h>
-#include <qdatetime.h>
+#include <QFileInfo>
+#include <QStringList>
+#include <QHostAddress>
+#include <QTimer>
+#include <QDateTime>
 #include <kcodecs.h>
 #include <kapplication.h>
 #include <kgenericfactory.h>
@@ -30,26 +35,30 @@
 #include <k3streamsocket.h>
 #include <k3resolver.h>
 
-#include <qfileinfo.h>
-#include <qstringlist.h>
-
 #include <interfaces/coreinterface.h>
 #include <interfaces/torrentinterface.h>
-
 #include <util/log.h>		
 #include <util/fileops.h>
 #include <util/functions.h>
 #include <util/mmapfile.h>
 #include <util/sha1hash.h>
-#include <time.h>
 #include "ktversion.h"
 #include "httpserver.h"
 #include "httpclienthandler.h"
 #include "httpresponseheader.h"
-#include "phphandler.h"
-#include "phpcommandhandler.h"
-#include "phpcodegenerator.h"
 #include "webinterfacepluginsettings.h"
+#include "torrentlistgenerator.h"
+#include "challengegenerator.h"
+#include "loginhandler.h"
+#include "logouthandler.h"
+#include "actionhandler.h"
+#include "iconhandler.h"
+#include "torrentposthandler.h"
+#include "torrentfilesgenerator.h"
+#include "globaldatagenerator.h"
+#include "settingsgenerator.h"
+
+
 
 using namespace bt;
 
@@ -58,46 +67,74 @@ namespace kt
 	QString DataDir();
 	
 
-	HttpServer::HttpServer(CoreInterface *core, bt::Uint16 port) : sock(0),notifier(0),core(core),cache(10),port(port)
+	HttpServer::HttpServer(CoreInterface *core, bt::Uint16 port) : core(core),cache(10),port(port)
 	{
 		qsrand(time(0));
-		sock = new net::Socket(true,4);
+		content_generators.setAutoDelete(true);
+		addContentGenerator(new TorrentListGenerator(core,this));
+		addContentGenerator(new ChallengeGenerator(this));
+		addContentGenerator(new LoginHandler(this));
+		addContentGenerator(new LogoutHandler(this));
+		addContentGenerator(new ActionHandler(core,this));
+		addContentGenerator(new TorrentPostHandler(core,this));
+		addContentGenerator(new TorrentFilesGenerator(core,this));
+		addContentGenerator(new IconHandler(this));
+		addContentGenerator(new GlobalDataGenerator(core,this));
+		addContentGenerator(new SettingsGenerator(core,this));
 
-		php_cmd = new PhpCommandHandler(core);
-		php_gen = new PhpCodeGenerator(core);
-
-		
 		QStringList dirList = KGlobal::dirs()->findDirs("data", "ktorrent/www");
-		rootDir = dirList.front();
-		Out(SYS_WEB|LOG_DEBUG) << "WWW Root Directory "<< rootDir <<endl;
+		if (!dirList.empty())
+		{
+			rootDir = dirList.front();
+			Out(SYS_WEB|LOG_DEBUG) << "WWW Root Directory "<< rootDir <<endl;
+		}
 		session.logged_in = false;
 		
-		ok = sock->bind(QString::null,port,true);
-		if (ok)
+
+		QStringList bind_addresses;
+		bind_addresses << QHostAddress(QHostAddress::Any).toString();
+		bind_addresses << QHostAddress(QHostAddress::AnyIPv6).toString();
+		foreach (const QString & addr,bind_addresses)
 		{
-			notifier = new QSocketNotifier(sock->fd(),QSocketNotifier::Read,this);
-			connect(notifier,SIGNAL(activated(int)),this,SLOT(slotAccept(int)));
+			net::ServerSocket::Ptr sock(new net::ServerSocket(this));
+			if (sock->bind(addr,port))
+				sockets.append(sock);
 		}
 
-		skin_list = QDir(rootDir).entryList(QDir::Dirs);
-		skin_list.removeAll("common");
-		skin_list.removeAll(".");
-		skin_list.removeAll("..");
+		if (!rootDir.isEmpty())
+		{
+			skin_list = QDir(rootDir).entryList(QDir::Dirs);
+			skin_list.removeAll("common");
+			skin_list.removeAll(".");
+			skin_list.removeAll("..");
+			foreach (QString s,skin_list)
+				Out(SYS_WEB|LOG_DEBUG) << "skin: " << s << endl;
+		}
 	}
 	
 	HttpServer::~HttpServer()
 	{
-		sock->close();
-		delete sock;
-		delete php_cmd;
-		delete php_gen;
+		qDeleteAll(clients);
 	}
 
 	QString HttpServer::skinDir() const
 	{
-		QString skin = skin_list[WebInterfacePluginSettings::skin()];
-		if (skin.length() == 0)
+		QString skin;
+		if (skin_list.count() == 0)
+		{
 			skin = "default";
+		}
+		else
+		{
+			int s = WebInterfacePluginSettings::skin();
+			if (s < 0 || s >= skin_list.count())
+				s = 0;
+		
+			skin = skin_list.at(s);
+			if (skin.length() == 0)
+				skin = "default";
+		}
+			
 		return rootDir + bt::DirSeparator() + skin;
 	}
 	
@@ -105,41 +142,32 @@ namespace kt
 	{
 		return rootDir + bt::DirSeparator() + "common";
 	}
-
-	void HttpServer::slotAccept(int )
-	{
-		net::Address addr;
-		int socket = sock->accept(addr);
-		if (socket < 0)
-			return;
 	
-		HttpClientHandler* handler = new HttpClientHandler(this,socket);
+	void HttpServer::newConnection(int fd, const net::Address& addr)
+	{
+		HttpClientHandler* handler = new HttpClientHandler(this,fd);
 		connect(handler,SIGNAL(closed()),this,SLOT(slotConnectionClosed()));
 		Out(SYS_WEB|LOG_NOTICE) << "connection from "<< addr.toString()  << endl;
+		clients.append(handler);
 	}
-/*
-	static int DecodeEscapedChar(QString & password,int idx)
-	{
-		QChar a = password[idx + 1].toLower();
-		QChar b = password[idx + 2].toLower();
-		if (!a.isNumber() && !(a.toLatin1() >= 'a' && a.toLatin1() <= 'f'))
-			return idx + 2; // not a valid hex digit
-		
-		if (!b.isNumber() && !(b.toLatin1() >= 'a' && b.toLatin1() <= 'f'))
-			return idx + 2; // not a valid hex digit
-		
-		// calculate high and low nibble
-		Uint8 h = (a.toLatin1() - (a.isNumber() ? '0' : 'a')) << 4;
-		Uint8 l = (b.toLatin1() - (b.isNumber() ? '0' : 'a'));
-		char r = (char) h | l; // combine them and cast to a char
-		password.replace(idx,3,r);
-		return idx + 1;
-	}
-	*/
+
+
 	bool HttpServer::checkLogin(const QHttpRequestHeader & hdr,const QByteArray & data)
 	{
+		// Authentication is disabled
+		if (!WebInterfacePluginSettings::authentication())
+		{
+			session.logged_in = true;
+			session.sessionId = rand();
+			session.last_access = QTime::currentTime();
+			Out(SYS_WEB|LOG_NOTICE) << "Webgui login succesfull ! (auth disable)" << endl;
+			challenge = QString();
+			return true;
+		}
+
 		if (hdr.contentType() != "application/x-www-form-urlencoded")
 		{
+			Out(SYS_WEB|LOG_NOTICE) << "Webgui login failed ! 1" << endl;
 			challenge = QString();
 			return false;
 		}
@@ -158,6 +186,7 @@ namespace kt
 
 		if (username.isNull() || challenge.isNull() || username != WebInterfacePluginSettings::username())
 		{
+			Out(SYS_WEB|LOG_NOTICE) << "Webgui login failed ! 2" << endl;
 			challenge = QString();
 			return false;
 		}
@@ -174,9 +203,19 @@ namespace kt
 			return true;
 		}
 		challenge = QString();
+		Out(SYS_WEB|LOG_NOTICE) << "Webgui login failed ! 3" << endl;
 		return false;
 	}
 	
+	
+	void HttpServer::logout()
+	{
+		session.logged_in = false;
+		session.sessionId = 0;
+		challenge = QString();
+		Out(SYS_WEB|LOG_NOTICE) << "Webgui logout" << endl;
+	}
+	
 	bool HttpServer::checkSession(const QHttpRequestHeader & hdr)
 	{
 		// check session in cookie
@@ -184,23 +223,16 @@ namespace kt
 		if (hdr.hasKey("Cookie"))
 		{
 			QString cookie = hdr.value("Cookie");
-			int idx = cookie.indexOf("KT_SESSID=");
-			if (idx == -1)
-				return false;
+			QRegExp rx("KT_SESSID=(\\d+)",Qt::CaseInsensitive);
+			int pos = 0;
 			
-			QString number;
-			idx += QString("KT_SESSID=").length();
-			while (idx < cookie.length())
+			while ((pos = rx.indexIn(cookie, pos)) != -1) 
 			{
-				if (cookie[idx] >= '0' && cookie[idx] <= '9')
-					number += cookie[idx];
-				else
+				session_id = rx.cap(1).toInt();
+				if (session_id == session.sessionId)
 					break;
-				
-				idx++;
+				pos += rx.matchedLength();
 			}
-					
-			session_id = number.toInt();
 		}
 		
 	
@@ -232,6 +264,8 @@ namespace kt
 			return "text/javascript";
 		else if (ext == "gif" || ext == "png" || ext == "ico")
 			return "image/" + ext;
+		else if (ext == "jpg" || ext == "jpeg")
+			return "image/jpeg";
 		else
 			return QString::null;
 	}
@@ -263,8 +297,9 @@ namespace kt
 	{
 		hdr.setValue("Server","KTorrent/" KT_VERSION_MACRO);
 		hdr.setValue("Date",DateTimeToString(QDateTime::currentDateTime().toUTC(),false));
-		hdr.setValue("Content-Type",content_type);
-		hdr.setValue("Connection","keep-alive");
+		if (!content_type.isEmpty())
+			hdr.setValue("Content-Type",content_type);
+		
 		if (with_session_info && session.sessionId && session.logged_in)
 		{
 			hdr.setValue("Set-Cookie",QString("KT_SESSID=%1").arg(session.sessionId));
@@ -273,75 +308,91 @@ namespace kt
 	
 	void HttpServer::redirectToLoginPage(HttpClientHandler* hdlr)
 	{
-		HttpResponseHeader rhdr(301);
+		HttpResponseHeader rhdr(302);
 		setDefaultResponseHeaders(rhdr,"text/html",false);
-		rhdr.setValue("Location","/login.html");
-		QString path = skinDir() + "/login.html";
-		if (!hdlr->sendFile(rhdr,path))
-		{
-			HttpResponseHeader nhdr(404);
-			setDefaultResponseHeaders(nhdr,"text/html",false);
-			hdlr->send404(nhdr,path);
-		}
+		rhdr.setValue("Location","login.html");
+		rhdr.setValue("Content-Length","0");
+		hdlr->sendResponse(rhdr);
 		Out(SYS_WEB|LOG_NOTICE) << "Redirecting to /login.html" << endl;
 	}
 	
-	void HttpServer::handleGet(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr,bool do_not_check_session)
+	
+	
+	void HttpServer::handleGet(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr)
 	{
+		if (rootDir.isEmpty())
+		{
+			HttpResponseHeader rhdr(500,hdr.majorVersion(),hdr.minorVersion());
+			setDefaultResponseHeaders(rhdr,"text/html",false);
+			hdlr->send500(rhdr,i18n("Cannot find web interface skins."));
+			return;
+		}
+		
 		QString file = hdr.path();
-		if (file == "/")
+		if (file == "/" && WebInterfacePluginSettings::authentication())
 			file = "/login.html";
-		
-		//Out(SYS_WEB|LOG_DEBUG) << "GET " << hdr.path() << endl;
-		
+		else if (file == "/")
+			file = "/interface.html";
+			
 		KUrl url;
 		url.setEncodedPathAndQuery(file);
 		
-		QString path = commonDir() + url.path();
-		// first try the common dir
-		if (!bt::Exists(path)) // doesn't exist so it must be in the skin dir
-			path = skinDir() + url.path();
-		
+		Out(SYS_WEB|LOG_DEBUG) << "GET " << hdr.path() << endl;
+		WebContentGenerator* gen = content_generators.find(url.path());
+		if (gen)
+		{
+			if ((gen->getPermissions() == WebContentGenerator::LOGIN_REQUIRED && (!session.logged_in || !checkSession(hdr))) && WebInterfacePluginSettings::authentication())
+			{
+				// redirect to login page
+				redirectToLoginPage(hdlr);
+			}
+			else
+			{
+				gen->get(hdlr,hdr);
+			}
+		}
+		else
+		{
+			QString path = commonDir() + url.path();
+			// first try the common dir
+			if (!bt::Exists(path)) // doesn't exist so it must be in the skin dir
+				path = skinDir() + url.path();
+			
+			handleFile(hdlr,hdr,path);
+		}
+	}
+	
+	void HttpServer::handleFile(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr,const QString & path)
+	{
 		// check if the file exists (if not send 404)
 		if (!bt::Exists(path))
 		{
-			HttpResponseHeader rhdr(404);
+			HttpResponseHeader rhdr(404,hdr.majorVersion(),hdr.minorVersion());
 			setDefaultResponseHeaders(rhdr,"text/html",false);
 			hdlr->send404(rhdr,path);
 			return;
 		}
 		
-		QFileInfo fi(path);
-		QString ext = fi.suffix();
+		QString file = hdr.path();
+		if (file == "/" && WebInterfacePluginSettings::authentication())
+			file = "/login.html";
+		else if (file == "/")
+			file = "/interface.html";
 		
-		// if it is the login page send that
-		if (file == "/login.html" || file == "/")
+		QFileInfo fi(path);
+		QString ext = fi.suffix();;
+
+		if (ext == "html")
 		{
-			session.logged_in = false;
-			ext = "html";
-			path = skinDir() + "/login.html"; 
-		}
-		else if (!session.logged_in && (ext == "html" || ext == "php"))
-		{
-			// for any html or php page, a login is necessary
-			redirectToLoginPage(hdlr);
-			return;
-		}
-		else if (session.logged_in && !do_not_check_session && (ext == "html" || ext == "php"))
-		{
-			// if we are logged in and it's a html or php page, check the session id
-			if (!checkSession(hdr))
+			// html pages require a login unless it is the login.html page
+			if ((file != "/login.html" && (!session.logged_in || !checkSession(hdr))) && WebInterfacePluginSettings::authentication())
 			{
-				session.logged_in = false;
 				// redirect to login page
 				redirectToLoginPage(hdlr);
 				return;
 			}
-		}
-		
-		if (ext == "html")
-		{
-			HttpResponseHeader rhdr(200);
+			
+			HttpResponseHeader rhdr(200,hdr.majorVersion(),hdr.minorVersion());
 			setDefaultResponseHeaders(rhdr,"text/html",true);
 			if (path.endsWith("login.html"))
 			{
@@ -353,155 +404,98 @@ namespace kt
 			
 			if (!hdlr->sendFile(rhdr,path))
 			{
-				HttpResponseHeader nhdr(404);
+				HttpResponseHeader nhdr(404,hdr.majorVersion(),hdr.minorVersion());
 				setDefaultResponseHeaders(nhdr,"text/html",false);
 				hdlr->send404(nhdr,path);
 			}
 		}
 		else if (ext == "css" || ext == "js" || ext == "png" || ext == "ico" || ext == "gif" || ext == "jpg")
 		{
-			if (hdr.hasKey("If-Modified-Since"))
-			{
-				QDateTime dt = parseDate(hdr.value("If-Modified-Since"));
-				if (dt.isValid() && dt < fi.lastModified())
-				{	
-					HttpResponseHeader rhdr(304);
-					setDefaultResponseHeaders(rhdr,"text/html",true);
-					rhdr.setValue("Cache-Control","max-age=0");
-					rhdr.setValue("Last-Modified",DateTimeToString(fi.lastModified(),false));
-					rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime().toUTC().addSecs(3600),false));
-					hdlr->sendResponse(rhdr);
-					return;
-				}
-			}
-			
-			
-			HttpResponseHeader rhdr(200);
-			setDefaultResponseHeaders(rhdr,ExtensionToContentType(ext),true);
-			rhdr.setValue("Last-Modified",DateTimeToString(fi.lastModified(),false));
-			rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime().toUTC().addSecs(3600),false));
-			rhdr.setValue("Cache-Control","private");
-			if (!hdlr->sendFile(rhdr,path))
-			{
-				HttpResponseHeader nhdr(404);
-				setDefaultResponseHeaders(nhdr,"text/html",false);
-				hdlr->send404(nhdr,path);
-			}
-		}
-		else if (ext == "php")
-		{
-			QMap<QString,QString> args = url.queryItems();
-			bool redirect = false;
-			bool shutdown = false;
-			if (args.count() > 0 && session.logged_in)
-				redirect = php_cmd->exec(url,shutdown);
-			
-			if (shutdown)
-			{
-				// first send back login page
-				redirectToLoginPage(hdlr);
-				QTimer::singleShot(1000,kapp,SLOT(quit()));
-			}
-			else if (redirect)
-			{
-				HttpResponseHeader rhdr(301);
-				setDefaultResponseHeaders(rhdr,"text/html",true);
-				rhdr.setValue("Location",url.encodedPathAndQuery());
-				
-				hdlr->executePHPScript(php_gen,rhdr,WebInterfacePluginSettings::phpExecutablePath().path(),
-									   path,url.queryItems());
-			}
-			else
-			{
-				HttpResponseHeader rhdr(200);
-				setDefaultResponseHeaders(rhdr,"text/html",true);
-			
-				hdlr->executePHPScript(php_gen,rhdr,WebInterfacePluginSettings::phpExecutablePath().path(),
-								   path,url.queryItems());
-			}
+			handleNormalFile(hdlr,hdr,path);
 		}
 		else
 		{
-			HttpResponseHeader rhdr(404);
+			HttpResponseHeader rhdr(404,hdr.majorVersion(),hdr.minorVersion());
 			setDefaultResponseHeaders(rhdr,"text/html",false);
-			hdlr->send404(rhdr,path);
+			hdlr->send404(rhdr,file);
+		}
+	}
+	
+	void HttpServer::handleNormalFile(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr,const QString & path)
+	{
+		QFileInfo fi(path);
+		QString ext = fi.suffix();
+		
+		if (hdr.hasKey("If-Modified-Since"))
+		{
+			QDateTime dt = parseDate(hdr.value("If-Modified-Since"));
+			if (dt.isValid() && dt < fi.lastModified())
+			{	
+				HttpResponseHeader rhdr(304,hdr.majorVersion(),hdr.minorVersion());
+				setDefaultResponseHeaders(rhdr,"text/html",true);
+				rhdr.setValue("Cache-Control","max-age=0");
+				rhdr.setValue("Last-Modified",DateTimeToString(fi.lastModified(),false));
+				rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime().toUTC().addSecs(3600),false));
+				hdlr->sendResponse(rhdr);
+				return;
+			}
+		}
+			
+		HttpResponseHeader rhdr(200,hdr.majorVersion(),hdr.minorVersion());
+		setDefaultResponseHeaders(rhdr,ExtensionToContentType(ext),true);
+		rhdr.setValue("Last-Modified",DateTimeToString(fi.lastModified(),false));
+		rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime().toUTC().addSecs(3600),false));
+		rhdr.setValue("Cache-Control","private");
+		if (!hdlr->sendFile(rhdr,path))
+		{
+			HttpResponseHeader nhdr(404,hdr.majorVersion(),hdr.minorVersion());
+			setDefaultResponseHeaders(nhdr,"text/html",false);
+			hdlr->send404(nhdr,path);
 		}
 	}
 	
 	void HttpServer::handlePost(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr,const QByteArray & data)
 	{
-		// this is either a file or a login
-		if (hdr.value("Content-Type").startsWith("multipart/form-data"))
+		Out(SYS_WEB|LOG_DEBUG) << "POST " << hdr.path() << endl;
+		KUrl url;
+		url.setEncodedPathAndQuery(hdr.path());
+		WebContentGenerator* gen = content_generators.find(url.path());
+		if (gen)
 		{
-			handleTorrentPost(hdlr,hdr,data);
-		}
-		else if (!checkLogin(hdr,data))
-		{
-			QHttpRequestHeader tmp = hdr;
-			tmp.setRequest("GET","/login.html",1,1);
-			handleGet(hdlr,tmp);
+			if ((gen->getPermissions() == WebContentGenerator::LOGIN_REQUIRED && (!session.logged_in || !checkSession(hdr))) && WebInterfacePluginSettings::authentication())
+			{
+				// redirect to login page
+				redirectToLoginPage(hdlr);
+			}
+			else
+			{
+				gen->post(hdlr,hdr,data);
+			}
 		}
 		else
 		{
-			handleGet(hdlr,hdr,true);
+			KUrl url;
+			url.setEncodedPathAndQuery(hdr.path());
+			QString path = commonDir() + url.path();
+			// first try the common dir
+			if (!bt::Exists(path)) // doesn't exist so it must be in the skin dir
+				path = skinDir() + url.path();
+			
+			handleFile(hdlr,hdr,path);
 		}
 	}
 	
-	void HttpServer::handleTorrentPost(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr,const QByteArray & data)
+	void HttpServer::handleUnsupportedMethod(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr)
 	{
-		const char* ptr = data.data();
-		int len = data.size();
-		int pos = QString(data).indexOf("\r\n\r\n");
-		
-		if (!session.logged_in || !checkSession(hdr))
-		{
-			// You can't post torrents if you are not logged in
-			// or the session is not OK
-			redirectToLoginPage(hdlr);
-			return;
-		}
-		
-		if (pos == -1 || pos + 4 >= len)
-		{
-			HttpResponseHeader rhdr(500);
-			setDefaultResponseHeaders(rhdr,"text/html",false);
-			hdlr->send500(rhdr);
-			return;
-		}
-		
-		// save torrent to a temporary file
-		QString save_file = kt::DataDir() + "webgui_load_torrent";
-		QFile tmp_file(save_file);
-		
-		if (!tmp_file.open(QIODevice::WriteOnly))
-		{
-			HttpResponseHeader rhdr(500);
-			setDefaultResponseHeaders(rhdr,"text/html",false);
-			hdlr->send500(rhdr);
-			return;
-		}
-		
-		QDataStream out(&tmp_file);
-		out.writeRawData(ptr + (pos + 4),len - (pos + 4));
-		out << flush;
-		tmp_file.close();
-		
-		Out(SYS_WEB|LOG_NOTICE) << "Loading file " << save_file << endl;
-		core->loadSilently(KUrl(save_file),QString());
-		
-		handleGet(hdlr,hdr);
-	}
-	
-	void HttpServer::handleUnsupportedMethod(HttpClientHandler* hdlr)
-	{
-		HttpResponseHeader rhdr(500);
+		HttpResponseHeader rhdr(500,hdr.majorVersion(),hdr.minorVersion());
 		setDefaultResponseHeaders(rhdr,"text/html",false);
-		hdlr->send500(rhdr);
+		hdlr->send500(rhdr,i18n("Unsupported HTTP method"));
 	}
 	
 	void HttpServer::slotConnectionClosed()
 	{
 		HttpClientHandler* client = (HttpClientHandler*)sender();
+		clients.removeAll(client);
 		client->deleteLater();
 	}
 
@@ -596,6 +590,12 @@ namespace kt
 			challenge.append(RandomLetterOrNumber());
 		return challenge;
 	}
+	
+	void HttpServer::addContentGenerator(WebContentGenerator* g)
+	{
+		content_generators.insert(g->getPath(),g);
+	}
+	
 }
 
 #include "httpserver.moc"
